@@ -2,23 +2,17 @@
 
 """This module implements the combiner class."""
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import numpy as np
 from numpy import ma
-from .ccddata import CCDData
 from .core import sigma_func
 
-from astropy.nddata import StdDevUncertainty
+from astropy.nddata import CCDData, StdDevUncertainty
 from astropy import log
 
-import math
-
 __all__ = ['Combiner', 'combine']
 
 
-class Combiner(object):
+class Combiner:
     """
     A class for combining CCDData objects.
 
@@ -49,7 +43,8 @@ class Combiner(object):
 
         >>> import numpy as np
         >>> import astropy.units as u
-        >>> from ccdproc import Combiner, CCDData
+        >>> from astropy.nddata import CCDData
+        >>> from ccdproc import Combiner
         >>> ccddata1 = CCDData(np.ones((4, 4)), unit=u.adu)
         >>> ccddata2 = CCDData(np.zeros((4, 4)), unit=u.adu)
         >>> ccddata3 = CCDData(np.ones((4, 4)), unit=u.adu)
@@ -88,7 +83,7 @@ class Combiner(object):
                 default_unit = ccd.unit
             else:
                 if not (default_unit == ccd.unit):
-                    raise TypeError("CCDData objects don't the same unit.")
+                    raise TypeError("CCDData objects don't have the same unit.")
 
         self.ccd_list = ccd_list
         self.unit = default_unit
@@ -132,11 +127,12 @@ class Combiner(object):
     def weights(self, value):
         if value is not None:
             if isinstance(value, np.ndarray):
-                if value.shape == self.data_arr.data.shape:
-                    self._weights = value
-                else:
-                    raise ValueError(
-                        "dimensions of weights do not match data.")
+                if value.shape != self.data_arr.data.shape:
+                    if value.ndim != 1:
+                        raise ValueError("1D weights expected when shapes of the data and weights differ.")
+                    if value.shape[0] != self.data_arr.data.shape[0]:
+                        raise ValueError("Length of weights not compatible with specified axis.")
+                self._weights = value
             else:
                 raise TypeError("weights must be a numpy.ndarray.")
         else:
@@ -307,6 +303,13 @@ class Combiner(object):
             mask = (self.data_arr - baseline > high_thresh * dev)
             self.data_arr.mask[mask] = True
 
+    def _get_scaled_data(self, scale_arg):
+        if scale_arg is not None:
+            return self.data_arr * scale_arg
+        if self.scaling is not None:
+            return self.data_arr * self.scaling
+        return self.data_arr
+
     # set up the combining algorithms
     def median_combine(self, median_func=ma.median, scale_to=None,
                        uncertainty_func=sigma_func):
@@ -345,27 +348,17 @@ class Combiner(object):
         The uncertainty currently calculated using the median absolute
         deviation does not account for rejected pixels.
         """
-        if scale_to is not None:
-            scalings = scale_to
-        elif self.scaling is not None:
-            scalings = self.scaling
-        else:
-            scalings = 1.0
-
         # set the data
-        data = median_func(scalings * self.data_arr, axis=0)
+        data = median_func(self._get_scaled_data(scale_to), axis=0)
 
         # set the mask
         masked_values = self.data_arr.mask.sum(axis=0)
         mask = (masked_values == len(self.data_arr))
 
         # set the uncertainty
-        uncertainty = uncertainty_func(self.data_arr.data, axis=0)
+        uncertainty = uncertainty_func(self.data_arr, axis=0)
         # Divide uncertainty by the number of pixel (#309)
-        # TODO: This should be np.sqrt(len(self.data_arr) - masked_values) but
-        # median_absolute_deviation ignores the mask... so it
-        # would yield inconsistent results.
-        uncertainty /= math.sqrt(len(self.data_arr))
+        uncertainty /= np.sqrt(len(self.data_arr) - masked_values)
         # Convert uncertainty to plain numpy array (#351)
         # There is no need to care about potential masks because the
         # uncertainty was calculated based on the data so potential masked
@@ -415,15 +408,8 @@ class Combiner(object):
         combined_image: `~astropy.nddata.CCDData`
             CCDData object based on the combined input of CCDData objects.
         """
-        if scale_to is not None:
-            scalings = scale_to
-        elif self.scaling is not None:
-            scalings = self.scaling
-        else:
-            scalings = 1.0
-
         # set up the data
-        data, wei = scale_func(scalings * self.data_arr,
+        data, wei = scale_func(self._get_scaled_data(scale_to),
                                axis=0, weights=self.weights,
                                returned=True)
 
@@ -483,15 +469,8 @@ class Combiner(object):
         combined_image: `~astropy.nddata.CCDData`
             CCDData object based on the combined input of CCDData objects.
         """
-        if scale_to is not None:
-            scalings = scale_to
-        elif self.scaling is not None:
-            scalings = self.scaling
-        else:
-            scalings = 1.0
-
         # set up the data
-        data = sum_func(scalings * self.data_arr, axis=0)
+        data = sum_func(self._get_scaled_data(scale_to), axis=0)
 
         # set up the mask
         masked_values = self.data_arr.mask.sum(axis=0)
@@ -518,6 +497,63 @@ class Combiner(object):
         return combined_image
 
 
+def _calculate_step_sizes(x_size, y_size, num_chunks):
+    """
+    Calculate the strides in x and y to achieve at least
+    the ``num_chunks`` pieces.
+
+    Parameters
+    ----------
+    """
+    # First we try to split only along fast x axis
+    xstep = max(1, int(x_size / num_chunks))
+
+    # More chunks are needed only if xstep gives us fewer chunks than
+    # requested.
+    x_chunks = int(x_size / xstep)
+
+    if x_chunks >= num_chunks:
+        ystep = y_size
+    else:
+        # The x and y loops are nested, so the number of chunks
+        # is multiplicative, not additive. Calculate the number
+        # of y chunks we need to get at num_chunks.
+        y_chunks = int(num_chunks / x_chunks) + 1
+        ystep = max(1, int(y_size / y_chunks))
+
+    return xstep, ystep
+
+
+def _calculate_size_of_image(ccd,
+                             combine_uncertainty_function):
+    # If uncertainty_func is given for combine this will create an uncertainty
+    # even if the originals did not have one. In that case we need to create
+    # an empty placeholder.
+    if ccd.uncertainty is None and combine_uncertainty_function is not None:
+        ccd.uncertainty = StdDevUncertainty(np.zeros(ccd.data.shape))
+
+    size_of_an_img = ccd.data.nbytes
+    try:
+        size_of_an_img += ccd.uncertainty.array.nbytes
+    # In case uncertainty is None it has no "array" and in case the "array" is
+    # not a numpy array:
+    except AttributeError:
+        pass
+    # Mask is enforced to be a numpy.array across astropy versions
+    if ccd.mask is not None:
+        size_of_an_img += ccd.mask.nbytes
+    # flags is not necessarily a numpy array so do not fail with an
+    # AttributeError in case something was set!
+    # TODO: Flags are not taken into account in Combiner. This number is added
+    #       nevertheless for future compatibility.
+    try:
+        size_of_an_img += ccd.flags.nbytes
+    except AttributeError:
+        pass
+
+    return size_of_an_img
+
+
 def combine(img_list, output_file=None,
             method='average', weights=None, scale=None, mem_limit=16e9,
             clip_extrema=False, nlow=1, nhigh=1,
@@ -657,12 +693,6 @@ def combine(img_list, output_file=None,
         # User has provided fits filenames to read from
         ccd = CCDData.read(img_list[0], **ccdkwargs)
 
-    # If uncertainty_func is given for combine this will create an uncertainty
-    # even if the originals did not have one. In that case we need to create
-    # an empty placeholder.
-    if ccd.uncertainty is None and combine_uncertainty_function is not None:
-        ccd.uncertainty = StdDevUncertainty(np.zeros(ccd.data.shape))
-
     if dtype is None:
         dtype = np.float64
 
@@ -672,38 +702,28 @@ def combine(img_list, output_file=None,
     if ccd.data.dtype != dtype:
         ccd.data = ccd.data.astype(dtype)
 
-    size_of_an_img = ccd.data.nbytes
-    try:
-        size_of_an_img += ccd.uncertainty.array.nbytes
-    # In case uncertainty is None it has no "array" and in case the "array" is
-    # not a numpy array:
-    except AttributeError:
-        pass
-    # Mask is enforced to be a numpy.array across astropy versions
-    if ccd.mask is not None:
-        size_of_an_img += ccd.mask.nbytes
-    # flags is not necessarily a numpy array so do not fail with an
-    # AttributeError in case something was set!
-    # TODO: Flags are not taken into account in Combiner. This number is added
-    #       nevertheless for future compatibility.
-    try:
-        size_of_an_img += ccd.flags.nbytes
-    except AttributeError:
-        pass
+    size_of_an_img = _calculate_size_of_image(ccd,
+                                              combine_uncertainty_function)
 
     no_of_img = len(img_list)
 
+    # Set a memory use factor based on profiling
+    if method == 'median':
+        memory_factor = 3
+    else:
+        memory_factor = 2
+
+    memory_factor *= 1.5
+
     # determine the number of chunks to split the images into
-    no_chunks = int((size_of_an_img * no_of_img) / mem_limit) + 1
+    no_chunks = int((memory_factor * size_of_an_img * no_of_img) / mem_limit) + 1
     if no_chunks > 1:
         log.info('splitting each image into {0} chunks to limit memory usage '
                  'to {1} bytes.'.format(no_chunks, mem_limit))
     xs, ys = ccd.data.shape
-    # First we try to split only along fast x axis
-    xstep = max(1, int(xs/no_chunks))
-    # If more chunks need to be created we split in Y axis for remaining number
-    # of chunks
-    ystep = max(1, int(ys / (1 + no_chunks - int(xs / xstep))))
+
+    # Calculate strides for loop
+    xstep, ystep = _calculate_step_sizes(xs, ys, no_chunks)
 
     # Dictionary of Combiner properties to set and methods to call before
     # combining
@@ -759,11 +779,16 @@ def combine(img_list, output_file=None,
                 else:
                     imgccd = CCDData.read(image, **ccdkwargs)
 
-                # Trim image
-                ccd_list.append(imgccd[x:xend, y:yend])
+                # Trim image and copy
+                # The copy is *essential* to avoid having a bunch
+                # of unused file references around if the files
+                # are memory-mapped. See this PR for details
+                # https://github.com/astropy/ccdproc/pull/630
+                ccd_list.append(imgccd[x:xend, y:yend].copy())
 
             # Create Combiner for tile
             tile_combiner = Combiner(ccd_list, dtype=dtype)
+
             # Set all properties and call all methods
             for to_set in to_set_in_combiner:
                 setattr(tile_combiner, to_set, to_set_in_combiner[to_set])
@@ -783,6 +808,10 @@ def combine(img_list, output_file=None,
                 ccd.mask[x:xend, y:yend] = comb_tile.mask
             if ccd.uncertainty is not None:
                 ccd.uncertainty.array[x:xend, y:yend] = comb_tile.uncertainty.array
+            # Free up memory to try to stay under user's limit
+            del comb_tile
+            del tile_combiner
+            del ccd_list
 
     # Write fits file if filename was provided
     if output_file is not None:
