@@ -18,7 +18,6 @@ import warnings
 from astropy.utils.exceptions import AstropyUserWarning
 
 from .ccddata import fits_ccddata_reader, _recognized_fits_file_extensions
-from .tests.pytest_fixtures import directory_for_testing
 
 logger = logging.getLogger(__name__)
 
@@ -1001,14 +1000,3 @@ class ImageFileCollection:
         return self._generator('ccd', ccd_kwargs=ccd_kwargs, **kwd)
     ccds.__doc__ = _generator.__doc__.format(
         name='CCDData', default_scaling='True', return_type='astropy.nddata.CCDData')
-
-
-def sample_directory_with_files():
-    """
-    Returns the path to the small sample directory used
-    in the tests of ``ImageFileCollection``. Primarily intended
-    for use in the doctests.
-    """
-
-    n_test, tmpdir = directory_for_testing()
-    return tmpdir
