@@ -1,15 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 from functools import wraps
 import inspect
 from itertools import chain
 
 import numpy as np
 
-from astropy.extern import six
 from astropy.nddata import NDData
 from astropy import units as u
 from astropy.io import fits
@@ -102,18 +98,20 @@ def log_to_metadata(func):
             meta_dict = _metadata_to_dict(log_result)
         else:
             # Logging is not turned off, but user did not provide a value
-            # so construct one.
-            key = func.__name__
-            all_args = chain(zip(original_positional_args, args),
-                             six.iteritems(kwd))
-            all_args = ["{0}={1}".format(name,
-                                         _replace_array_with_placeholder(val))
-                        for name, val in all_args]
-            log_val = ", ".join(all_args)
-            log_val = log_val.replace("\n", "")
-            meta_dict = {key: log_val}
+            # so construct one unless the config parameter auto_logging is set to False
+            if ccdproc.conf.auto_logging:
+                key = func.__name__
+                all_args = chain(zip(original_positional_args, args), kwd.items())
+                all_args = ["{0}={1}".format(name,
+                                            _replace_array_with_placeholder(val))
+                            for name, val in all_args]
+                log_val = ", ".join(all_args)
+                log_val = log_val.replace("\n", "")
+                meta_dict = {key: log_val}
+            else:
+                meta_dict = {}
 
-        for k, v in six.iteritems(meta_dict):
+        for k, v in meta_dict.items():
             _insert_in_metadata_fits_safe(result, k, v)
         return result
 
@@ -121,7 +119,7 @@ def log_to_metadata(func):
 
 
 def _metadata_to_dict(arg):
-    if isinstance(arg, six.string_types):
+    if isinstance(arg, str):
         # add the key, no value
         return {arg: None}
     elif isinstance(arg, ccdproc.Keyword):
