@@ -1,14 +1,12 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import numpy as np
 import pytest
 
 import astropy.units as u
 
 from ..core import create_deviation, gain_correct, Keyword
+from .pytest_fixtures import ccd_data as ccd_data_func
 
 
 # tests for gain
@@ -17,8 +15,8 @@ from ..core import create_deviation, gain_correct, Keyword
                          3.0 * u.photon / u.adu,
                          3.0 * u.electron / u.adu,
                          Keyword('gainval', unit=u.electron / u.adu)])
-@pytest.mark.data_unit(u.adu)
-def test_linear_gain_correct(ccd_data, gain):
+def test_linear_gain_correct(gain):
+    ccd_data = ccd_data_func()
     # The data values should be positive, so the poisson noise calculation
     # works without throwing warnings
     ccd_data.data = np.absolute(ccd_data.data)
@@ -33,9 +31,9 @@ def test_linear_gain_correct(ccd_data, gain):
     except AttributeError:
         gain_value = gain
 
-    np.testing.assert_array_equal(ccd.data, gain_value * orig_data)
-    np.testing.assert_array_equal(ccd.uncertainty.array,
-                                  gain_value * ccd_data.uncertainty.array)
+    np.testing.assert_array_almost_equal_nulp(ccd.data, gain_value * orig_data)
+    np.testing.assert_array_almost_equal_nulp(
+        ccd.uncertainty.array, gain_value * ccd_data.uncertainty.array)
 
     if isinstance(gain, u.Quantity):
         assert ccd.unit == ccd_data.unit * gain.unit
@@ -44,8 +42,8 @@ def test_linear_gain_correct(ccd_data, gain):
 
 
 # test gain with gain_unit
-@pytest.mark.data_unit(u.adu)
-def test_linear_gain_unit_keyword(ccd_data):
+def test_linear_gain_unit_keyword():
+    ccd_data = ccd_data_func()
     # The data values should be positive, so the poisson noise calculation
     # works without throwing warnings
     ccd_data.data = np.absolute(ccd_data.data)
@@ -55,7 +53,7 @@ def test_linear_gain_unit_keyword(ccd_data):
     gain = 3.0
     gain_unit = u.electron / u.adu
     ccd = gain_correct(ccd_data, gain, gain_unit=gain_unit)
-    np.testing.assert_array_equal(ccd.data, gain * orig_data)
-    np.testing.assert_array_equal(ccd.uncertainty.array,
-                                  gain * ccd_data.uncertainty.array)
+    np.testing.assert_array_almost_equal_nulp(ccd.data, gain * orig_data)
+    np.testing.assert_array_almost_equal_nulp(
+        ccd.uncertainty.array, gain * ccd_data.uncertainty.array)
     assert ccd.unit == ccd_data.unit * gain_unit
