@@ -1,6 +1,8 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
+
 import numpy as np
 
-from astropy.tests.helper import pytest
+import pytest
 
 from ..slices import slice_from_string
 
